from enum import Enum


class ConfigKey(Enum):
    QUERENT_ID = "querent_id"
    COLLECTORS = "collectors"
    ENGINES = "engines"
    RESOURCE = "resource"
    VERSION = "version"
    QUERENT_NAME = "querent_name"
    WORKFLOW = "workflow"
    # Add more keys as needed
